/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc:fr.bigeon.gclc.ConsoleApplicationTest.java
 * Created on: Jun 9, 2016
 */
package fr.bigeon.gclc;

import static org.junit.Assert.fail;

import java.io.IOException;
import java.io.InputStream;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.PrintWriter;

import org.junit.Test;

import fr.bigeon.gclc.manager.SystemConsoleManager;

/** Test class for ConsoleApplication
 *
 * @author Emmanuel Bigeon */
@SuppressWarnings({"resource", "javadoc", "nls", "static-method"})
public class ConsoleApplicationTest {

    /** 3 seconds in milliseconds */
    protected static final long THREE_SECONDS = 3000;

    /** Test the base of a console application */
    @Test
    public void test() {
        ConsoleTestApplication app = new ConsoleTestApplication(
                new SystemConsoleManager());
        app.exit();
    }

    @Test
    public void executionTest() {
        try {
            CommandTestingApplication application = new CommandTestingApplication();

            application.sendCommand("test");
            application.sendCommand("toto");
            application.sendCommand("long");
            application.sendCommand("exit");

        } catch (IOException e1) {
            // TODO Auto-generated catch block
            e1.printStackTrace();
        }

        try {
            final PipedOutputStream src = new PipedOutputStream();
            InputStream in = new PipedInputStream(src);

            final ConsoleTestApplication app = new ConsoleTestApplication(
                    new SystemConsoleManager(System.out, in));
            Thread th = new Thread(new Runnable() {

                @Override
                public void run() {
                    app.start();
                }
            });

            th.start();

            Thread test = new Thread(new Runnable() {

                @Override
                public void run() {
                    try (PrintWriter writer = new PrintWriter(src, true)) {
                        writer.println("test");
                        writer.println("toto");
                        writer.println("long");
                        writer.println("exit");
                    }
                }
            });
            test.start();
            try {
                th.join();
            } catch (InterruptedException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
            try {
                test.join();
            } catch (InterruptedException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        } catch (IOException e) {
            fail("pipe creation"); //$NON-NLS-1$
        }
    }
}
